<html>

{% if notes %}
{% for note in notes %}
<h2>{{ note.name }}</h2>
<br>
<p>{{ note.description|safe }}</p>
<br>
{% if note.hasAlarm %}
<p>{{ alarm_i18n }} {{ note.alarm }}</p>
{% endif %}
<br>
{% endfor %}
{% endif %}

</html>

