<!DOCTYPE gui SYSTEM "kpartgui.dtd">

<gui name="KNotes" version="405" translationDomain="knotes">

<Menu name="knotes_context" noMerge="1"><text>KNotes</text>
    <title icon="knotes">KNotes</title>
    <Action name="new_note"/>
    <Action name="new_note_clipboard"/>
    <Action name="new_note_from_text_file"/>
    <Separator/>
    <Action name="edit_find"/>
    <Separator/>
    <Action name="delete_selected_notes"/>
    <Separator/>
    <Action name="print_selected_notes"/>
    <Separator/>
    <Action name="show_all_notes"/>
    <Action name="hide_all_notes"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure"/>
    <Separator/>
    <Menu name="help" icon="system-help" noMerge="1"><text>&amp;Help</text>
        <Action name="help_contents"/>
        <Separator/>
        <Action name="help_report_bug"/>
        <Separator/>
        <Action name="help_about_app"/>
        <Action name="help_about_kde"/>
    </Menu>
    <Separator/>
    <Action name="file_quit"/>
</Menu>

<Menu name="notes_menu" noMerge="1"><text>Notes</text>
    <title>Notes</title>
    <ActionList name="notes"/>
</Menu>

</gui>
