<!DOCTYPE gui>
<gui name="knotes" version="410" translationDomain="kontact">
  <MenuBar>
    <Menu name="file"><text>&amp;File</text>
      <Merge/>
      <Action name="new_note_clipboard"/>
      <Action name="new_note_from_text_file"/>
      <Separator/>
      <Action name="print_note"/>
      <Action name="print_preview_note"/>
    </Menu>
    <Menu name="edit"><text>&amp;Edit</text>
      <Action name="edit_note"/>
      <Action name="edit_rename"/>
      <Separator/>
      <Action name="edit_delete"/>
      <Separator/>
      <Action name="edit_find"/>
    </Menu>
    <Menu noMerge="1" name="settings">
     <text>&amp;Settings</text>
     <Action name="knotes_configure" group="settings_configure"/>
    </Menu>
  </MenuBar>
</gui>
